% Fuelwood extraction simulation model
% Authors: R. M. Holdo and W. C. Twine
% June 2016
% Note: this script uses the user-contributed functions ImportAsciiRaster.m
% and SaveAsciiRaster.m, as well as the rasters in files 'topography.txt',
% 'villages.txt' and 'precipitation.txt'. The model genherates a 'Results

% Matrices are capitalized, scalars and parameters are lowercase
close all
clear all

% USER-DEFINED MODEL VARIABLES -------------------------------------------
T = 50; % Total years in model run
mov = 0; % Movie output (Y=1/N=0)
finit=0.89; % proportion of households using firewood instead of electricity
% ------------------------------------------------------------------------

% Define model parameters-------------------------------------------------
sf = 6.25; % Scales up from ha to 250x250 m pixel
% Tree demographic parameters (first value is for lowlands, second for
% uplands)
alphax = 0.4; % Transition rate from X to S
alphac1 = 0.8; % Transition rate from C1 to C2
alphac2 = 0.2; % Transition rate from C2 to P
alphas = 0.1; % Transition rate from S to P
alphap = 0.05; % Transition rate from P to A
mu = 0.02; % Density-dependent parameter for resprout number
rho = 1; % Number of seedlings per adult
deltac = 0.5; % C1 mortality rate
deltaa = 0.01; % Adult mortality rate
sigmas = 9; % Number of resprouts per sapling
sigmap = 9; % Number of resprouts per pole
sigmaa = 9; % Number of resprouts per adult
beta1 = 0.11; % Topographic modifier for maximum AGB
beta2 = 0.0696; % Slope for max AGB vs. MAP relationship
beta3 = 26.387; % Intercept for max AGB vs. MAP relationship 
eta = 2.5*sf; % Maximum AGB annual increase
% Allometry parameters (Nickless et al. 2011)
gammax1=2.9e-5; gammax2=4.6e-5; % Conversion factors for lowland and upland seedlings
gammas1=0.6; gammas2=0.4; % Conversion factors for lowland and upland saplings
gammap1=14.3; gammap2=7.7;% Conversion factors for lowland and upland poles
gammaa1=367.5; gammaa2=149.6; % Conversion factors for lowland and upland adults
% Harvesting parameters
tau=16; % Max distance to collect firewood
phi=1; % Travel cost
FUEL=3028; % Fuelwood intake per household per year in kg
hhpv = [184 406 463 359 594 831 1284 696 623 445 1777 2138 1311 680 1113 700 459 811 631 1128]; % Households per village
epsilonS=0.64; epsilonP=2.89; epsilonA=3.11 % Preference ratios for saplings, poles and adults

% Load habitat, village, and rainfall GIS maps (in ASCII format - WGS 1984)
[topo, h] = ImportAsciiRaster(-9999,'topography.txt');
[village, h] = ImportAsciiRaster(-9999,'villages.txt');
nv=max(max(village)); % Number of villages
[rainfall, h] = ImportAsciiRaster(-9999,'precipitation.txt'); 
grid=size(village); % Number of rows and columns in map
xmax=grid(2); % number of columns
ymax=grid(1); % number of rows

% Initial conditions
xinit1=16899; xinit2=14235; % Initial numbers of seedlings by topography
sinit1=6470; sinit2=6303; % Initial numbers of saplings by topography
pinit1=683; pinit2=1039; % Initial numbers of poles by topography
ainit1=28; ainit2=69; % Initial mean numbers of adults by topography

col=colormap(jet(128));
% Initialize movie if necessary
if (mov)
    rect = get(gcf,'Position');
    movegui('center');
    rect(1:2) = [0 0];
    shot = 1;
    M(shot) = getframe(gcf);
end

% Output variable
out = zeros(T,10); % Output array

% Define and initialize spatial variables
Wd = zeros(ymax,xmax,nv); % Weight matrix by distance
Wb = zeros(ymax,xmax); % Weight matrix by biomass and stem ratios
SB = zeros(ymax,xmax,nv); % Sapling biomass target harvest for each village
PB = zeros(ymax,xmax,nv); % Pole biomass target harvest for each village
AB = zeros(ymax,xmax,nv); % Adult biomass target harvest for each village
Vharvest = zeros(T,nv); % Biomass harvest per village per year
meanS = zeros(T); % Mean saplings per year
meanP = zeros(T); % Mean poles per year
meanA = zeros(T); % Mean adults per year
meanAGB = zeros(T); % Mean AGB per year 
Fwood = ones(T+1,nv)*finit; % Fraction of each village using firewood instead of electricity
Vdemand=zeros(nv,1); % Biomass fuelwood demand per village
AGBmax = ((topo==1)*(1-beta1)+(topo==2)*(1+beta1)).*(beta2*rainfall-beta3)*sf; % Max AGB per pixel
allomX=(topo==1)*gammax1+(topo==2)*gammax2; % Allometry conversion factor for seedlings
allomS=(topo==1)*gammas1+(topo==2)*gammas2; % Allometry conversion factor for saplings
allomP=(topo==1)*gammap1+(topo==2)*gammap2; % Allometry conversion factor for poles 
allomA=(topo==1)*gammaa1+(topo==2)*gammaa2; % Allometry conversion factor for adults
% Ecological submodel variables
X = (xinit1*(topo==1)+xinit2*(topo==2))*sf/2;
C1 = X; % Coppiced "seedlings"
S = (sinit1*(topo==1)+sinit2*(topo==2))*sf;
C2 = zeros(ymax,xmax); % Coppiced saplings
P = (pinit1*(topo==1)+pinit2*(topo==2))*sf;
A = (ainit1*(topo==1)+ainit2*(topo==2))*sf; % Adults 
Hs = zeros(ymax,xmax); % Harvest of saplings per year
Hc = zeros(ymax,xmax); % Harvest of coppiced saplings per year
Hp = zeros(ymax,xmax); % Harvest of poles per year
Ha = zeros(ymax,xmax); % Harvest of adults per year

% Calculate the relative weight assigned to each cell according to distance
% to villages
for i=1:ymax, % Loop through rows (y)
    for j=1:xmax, % Loop through columns (x)
        if topo(i,j)>0 % Only use sites with habitat
            % Loop through harvest basin for each site
            for ii=i-tau:i+tau,
                for jj=j-tau:j+tau,
                    if (ii>=1 && ii<=ymax && jj>=1 && jj<=xmax) % Make sure we are still within the model lattice
                        d = sqrt((i-ii)^2+(j-jj)^2); % Calculate Euclidean distance
                        if (d<=tau && village(ii,jj)>0) % Are we within the threshold and is there a village here?
                            for k=1:nv, % Loop though villages
                                if (village(ii,jj)==k)
                                    %Wd(i,j,k)=Wd(i,j,k)+exp(-phi*d); % Calculate distance weight for village k at location i,j
                                    Wd(i,j,k)=Wd(i,j,k)+(tau-d)/(phi+tau-d); % Calculate distance weight for village k at location i,j
                                end
                            end
                        end
                    end
                end
            end
        end
    end
end

% Normalize village scores to sum to 1
for k=1:nv,
    vsum=sum(sum(Wd(:,:,k))); % Sum of weights for village k (should add up to 1)
    Wd(:,:,k)=Wd(:,:,k)/vsum; % This makes Wd add up to 1 for each village
end

for t=1:T,
    % Calculate village fuelwood demand
    for k=1:nv,
        Vdemand(k)=FUEL*Fwood(t,k)*hhpv(k); % Fuelwood demand based on village population size
    end
    % We now have cells weighted by distance to villages
    % Add the biomass component
    Wb = ((S+C2).*allomS*epsilonS + P.*allomP*epsilonP + A.*allomA*epsilonA).*(topo>0); % Relative expected biomass harvest, weighted by preference
    % Update correction factor for biomass weights each year
    for k=1:nv,
         % Sum biomass weights by village across harvest basin for each
         % village (i.e., all sites with Wd > 0)H
         Htot = Vdemand(k)*Wb.*Wd(:,:,k)/sum(sum(Wb.*Wd(:,:,k))); % Maximum harvest per village per cell
         nu = Htot./(Wb+(Wb==0));
         % Divide into sapling, pole and adult biomass
         SB(:,:,k) = nu.*(S+C2).*allomS*epsilonS; % Sapling biomass harvested per cell per village
         PB(:,:,k) = nu.*P.*allomP*epsilonP; % Pole biomass harvested per cell per village
         AB(:,:,k) = nu.*A.*allomA*epsilonA;
    end
    % Total biomass harvested per village before correcting for excessive
    % demand
    SBtot = sum(SB,3);
    PBtot = sum(PB,3);
    ABtot = sum(AB,3);
    % Correct for excessive demand by allocating what is available
    % proportionately
    OverdemandS = (SBtot - (S+C2).*allomS); % If +ve, there is more demand than supply of S
    OverdemandP = (PBtot - P.*allomP); % If +ve, there is more demand than supply of P
    OverdemandA = (ABtot - A.*allomA);
    % If there is excess demand, scale it back proportionately by village
    % If not, leave as is
    for k=1:nv,
        SB(:,:,k) = (OverdemandS > 0).*SB(:,:,k).*(S+C2).*allomS./(SBtot+(SBtot==0)) + (OverdemandS <= 0).*SB(:,:,k); 
        PB(:,:,k) = (OverdemandP > 0).*PB(:,:,k).*P.*allomP./(PBtot+(PBtot==0)) + (OverdemandP <= 0).*PB(:,:,k); 
        AB(:,:,k) = (OverdemandA > 0).*AB(:,:,k).*A.*allomA./(ABtot+(ABtot==0)) + (OverdemandA <= 0).*AB(:,:,k); 
        Vharvest(t,k) = sum(sum(SB(:,:,k)))+sum(sum(PB(:,:,k)))+sum(sum(AB(:,:,k)));
    end
    % Calculate number of stems removed per class by converting biomass to
    % stems
    Hs = sum(SB,3)./allomS.*S./(S+C2+(S+C2==0)); % Actual number of saplings harvested per cell
    Hc = sum(SB,3)./allomS.*C2./(S+C2+(S+C2==0)); % Actual number of coppiced saplings harvested per cell
    Hp = sum(PB,3)./allomP; % Actual number of poles harvested per cell 
    Ha = sum(AB,3)./allomA;
    % Ecological dynamics
    AGBsplus = ((S+C2).*allomS + P.*allomP + A.*allomA)/1e3; % AGB of S+P+A in t per cell
    AGBpplus = (P.*allomP + A.*allomA)/1e3; % AGB of P+A per cell
    AGBa = A.*allomA/1e3; % AGB of A per cell
    dds = max(0,(1-(AGBsplus./AGBmax).^2)); % Density dependent term for S
    ddp = max(0,(1-(AGBpplus./AGBmax).^2)); % Density dependent term for P
    dda = max(0,(1-(AGBa./AGBmax).^2)); % Density dependent term for A
    rx = alphax.*X.*dds; % Recruitment from X into S
    rs = alphas.*S.*ddp; % Recruitment from S into P
    rp = alphap.*P.*dda; % Recruitment from P into A
    rc1 = alphac1.*C1.*dds; % Recruitment from C1 into C2
    rc2 = alphac2.*C2.*ddp; % Recruitment from C2 into P
    x = rho.*A.*dds; % Seedling establishment (into X) from reproduction of A
    cs = (Hs+Hc).*sigmas;%.*exp(-mu.*C1); % Coppice stems produced from harvest of S and C2
    cp = Hp.*sigmap;%.*exp(-mu.*C1); % Coppice stems produced from harvest of P
    ca = Ha.*sigmaa;%.*exp(-mu.*C1); % Coppice stems produced from harvest of A
    mc = deltac.*(1-exp(-mu.*C1)).*C1; % Mortality of C1 as a result of crowding
    ma = deltaa.*A; % Adult mortality (senescence)
    % Potential change in AGB (dTot)
    dC1 = (cs + cp + ca - rc1).*allomX/1e3;
    dC2 = rc1.*allomS/1e3;
    dX = (x - rx).*allomX/1e3;
    dS = (rx - rs).*allomS/1e3;
    dP = (rs + rc2 - rp).*allomP/1e3;
    dA = rp.*allomA/1e3;   
    dTot = dC1 + dC2 + dX + dS + dP + dA;
    % Limit maximum AGB change to eta with a correction factor
    omega = (dTot > eta).*eta./dTot + (dTot <= eta)*1;
    % Update stem densities
    C1 = max(0,C1 + (cs + cp + ca - rc1).*omega - mc); % Update C1
    C2 = max(0,C2 + rc1.*omega - Hc); % Update C2
    X = max(0,X + (x - rx).*omega); % Update X
    S = max(0,S + (rx - rs).*omega - Hs); % Update S
    P = max(0,P + (rs + rc2 - rp).*omega - Hp); % Update P
    A = max(0,A + rp.*omega - ma - Ha); % Update A            
    % Update map of village by village harvest
    Vh = zeros(ymax,xmax); % Vharvest mapped in space
    for k=1:nv,
        Vh = Vh + (village==k)*Vharvest(t,k)/(hhpv(k)*1000);
    end
    % Plot annual changes in spatial woody dynamics
    if (mov)
        shot = shot+1;
        subplot(2,2,1)
        contourf((C2+S)/sf,'-k');
        axis square;
        xlabel('X coordinate');
        ylabel('Y coordinate');
        title('Saplings/ha');
        subplot(2,2,2)
        contourf(P/sf,'-k');
        axis square;
        xlabel('X coordinate');
        ylabel('Y coordinate');
        title('Poles/ha');
        subplot(2,2,3)
        contourf(A/sf,'-k');
        axis square;
        xlabel('X coordinate');
        ylabel('Y coordinate');
        title('Adults/ha');
        subplot(2,2,4)
        contourf((sum(PB,3)+sum(SB,3))/(1000*sf),'-k');
        axis square;
        xlabel('X coordinate');
        ylabel('Y coordinate');
        title('Harvest (t/ha)');
        set(gcf, 'Color', 'w');
        M(shot) = getframe(gcf,rect);
    end
    % Update output array
    out(t,1) = finit;
    out(t,2) = t;
    out(t,4) = sum(sum(AGBsplus))/(sf*sum(sum(topo>0)));
    out(t,5) = sum(sum(X))/(sf*sum(sum(topo>0))); % mean seedling density (ha-1)
    out(t,6) = sum(sum(C1))/(sf*sum(sum(topo>0))); % mean small coppiced stem density (ha-1)
    out(t,7) = sum(sum(S))/(sf*sum(sum(topo>0))); % mean sapling density (ha-1)
    out(t,8) = sum(sum(C2))/(sf*sum(sum(topo>0))); % mean large coppiced stem density (ha-1)
    out(t,9) = sum(sum(P))/(sf*sum(sum(topo>0))); % mean pole density (ha-1)
    out(t,10) = sum(sum(A))/(sf*sum(sum(topo>0))); % mean adult density (ha-1)
end
out(:,3) = sum(Vharvest,2)/1000; % village harvest per year
meanS = out(:,6)+out(:,7);
meanP = out(:,9);
meanA = out(:,10);
meanAGB = out(:,4); % mean AGB (t ha-1)
% Save movie
if (mov)
    movie2avi(M,'Fuelwood_movie.avi','fps',5,'quality',100);
else
    subplot(2,2,1)
    contourf((C2+S)/sf,'-k');
    axis square;
    xlabel('X coordinate');
    ylabel('Y coordinate');
    title('Saplings/ha');
    subplot(2,2,2)
    contourf(P/sf,'-k');
    axis square;
    xlabel('X coordinate');
    ylabel('Y coordinate');
    title('Poles/ha');
    subplot(2,2,3)
    contourf(C2./S,'-k');
    axis square;
    xlabel('X coordinate');
    ylabel('Y coordinate');
    title('Coppice:sapling ratio');
    subplot(2,2,4)
    contourf(A/sf,'-k');
    axis square;
    xlabel('X coordinate');
    ylabel('Y coordinate');
    title('Adults/ha');
    figure
    subplot(1,2,1)
    contourf(Vh,'-k');
    axis square;
    xlabel('X coordinate');
    ylabel('Y coordinate');
    title('Harvest (t/hh)');
    subplot(1,2,2)
    contourf((sum(PB,3)+sum(SB,3)+sum(AB,3))/(1000*sf),'-k');
    axis square;
    xlabel('X coordinate');
    ylabel('Y coordinate');
    title('Harvest (t/ha)');   
end
% Plot harvest temporal dynamics
time=1:T;
figure
subplot(3,2,1)
plot(time,sum(Vharvest,2)/1000)
xlabel('Time (y)')
ylabel('Total harvest (t)')
ylim([0 1.2*max(sum(Vharvest,2))/1000]);
subplot(3,2,2)
plot(time,meanAGB)
xlabel('Time (y)')
ylabel('AGB (t/ha)')
subplot(3,2,3)
plot(time,meanS)
xlabel('Time (y)')
ylabel('Saplings/ha')
subplot(3,2,4)
plot(time,meanP)
xlabel('Time (y)')
ylabel('Poles/ha')
subplot(3,2,5)
plot(time,meanA)
xlabel('Time (y)')
ylabel('Adults/ha')

% Plot final size distribution data
figure
N=zeros(4,2);
N(1,1)=sum(sum(X))/(sf*sum(sum(topo>0)));
N(2,1)=sum(sum(S))/(sf*sum(sum(topo>0))); 
N(3,1)=sum(sum(P))/(sf*sum(sum(topo>0))); 
N(4,1)=sum(sum(A))/(sf*sum(sum(topo>0)));  
N(1,2)=sum(sum(C1))/(sf*sum(sum(topo>0))); 
N(2,2)=sum(sum(C2))/(sf*sum(sum(topo>0))); 
bar(N,'stacked')
xlabel('Size class')
ylabel('Frequency')
legend('undamaged','coppiced')

% Output results to text file
save('Results.txt','out','-ASCII');

% Export final GIS layers to ArcMAP ASCII file
Vhrast = Vh + (village == -9999)*(-9999);
SaveAsciiRaster(Vhrast, h);
Harvrast = (sum(PB,3)+sum(SB,3)+sum(AB,3))/(1000*sf) + (topo == -9999)*(-9999);
SaveAsciiRaster(Harvrast, h);
AGBrast = AGBsplus/sf + (topo == -9999)*(-9999);
SaveAsciiRaster(AGBrast, h);
Srast = (S+C2)/sf + (topo == -9999)*(-9999);
SaveAsciiRaster(Srast, h);
Prast = P/sf + (topo == -9999)*(-9999);
SaveAsciiRaster(Prast, h);
Arast = A/sf + (topo == -9999)*(-9999);
SaveAsciiRaster(Arast, h);